import os
import pandas as pd
import sys
import logging
def main():
    if len(sys.argv) !=5:
        print("Error : Number of paramters are not correct")
        print("Usage : python program.py inputfile.csv '1,1,1,1' '+,+,-,+' result.csv ")
        exit(1)
        
        #File not found error 
    elif not os.path.isfile(sys.argv[1]):
        print(f"Error : {sys.argv[1]} Don't exist!")
        exit(1)
            
         #File not of csv extension   
    elif ".csv" != (os.path.splitext(sys.argv[1]))[1]:
        print(f"Error : {sys.argv[1]} is not csv file !")
        exit(1)
        
    else:
        dataset=pd.read_csv(sys.argv[1])
        temp_data=pd.read_csv(sys.argv[1])
        col=len(temp_data.columns.values) #no of columns
        
        #less than 3 columns in input file
        if col <3:
            print("Error: Input file have less than 3 columns")
            exit(1)
            
        #Handeling non-numerical values    
        for i in range(1,col):
            pd.to_numeric(dataset.iloc[:,i],errors='coerce')
            dataset.iloc[:,i].fillna((dataset.iloc[:,i].mean()), inplace=True)
            
        #Handling errors of weights and impact
        if ',' not in sys.argv[2]:
            logging.error("Weights should be separated by ','")
            return
        try:
            weights=[int(i) for i in sys.argv[2].split(',')]
        except:
            print("Error: In weights array please check once")
            exit(1)
        
        
        impact = sys.argv[3].split(',')
        for i in impact:
            if not (i== '+' or i=='-'):
                print("Error: In impact array please check once,it must contain '+' or '-'")
                exit(1)
        if ',' not in sys.argv[3]:
            logging.error("Impacts should be separated by ','")
            return
        
        #Checking number of columns,weights and impacts
        if col != len(weights)+1 or col != len(impact)+1:
            print("ERROR : Number of weights, number of impacts and number of columns are not same")
            exit(1)
        if (".csv" != (os.path.splitext(sys.argv[4]))[1]):
            print("ERROR: Output file extension is wrong")
            exit(1)
        if os.path.isfile(sys.argv[4]):
            os.remove(sys.argv[4])
        topsis_pipy(temp_data,dataset,col,weights,impact)
        
        
            
            
           
            
#Normalising the array
def Normalize(temp_data,col,weights):
    for i in range(1,col):
        temp=0
        for j in range(len(temp_data)): #no of rows
            temp= temp +temp_data.iloc[j,i]**2 #sum of squares
        temp=temp**0.5
        for j in range(len(temp_data)):
            temp_data.iloc[j,i]=(temp_data.iloc[j,i]/temp)*weights[i-1]
    return temp_data
#Calculating positive and negative values
def calc(temp_data,col,impact):
    p_val=(temp_data.max().values)[1:]
    n_val=(temp_data.min().values)[1:]
    for i in range(1,col):
        if impact[i-1] == '-':
            p_val[i-1],n_val[i-1] = n_val[i-1],p_val[i-1]
    return p_val,n_val

def topsis_pipy(temp_data,dataset,col,weights,impact):
    #Normalising the array
    #print(temp_data)
    temp_data=Normalize(temp_data,col,weights)
    #Calculating positive and negative values
    p_val,n_val=calc(temp_data,col,impact)
    
    #Calculating topsis score
    finalscore=[]
    for i in range(len(temp_data)):
        temp_p,temp_n=0,0
        for j in range(1,col):
            temp_p=temp_p+(p_val[j-1]-temp_data.iloc[i,j])**2
            temp_n=temp_n+(n_val[j-1]-temp_data.iloc[i,j])**2
        temp_p,temp_n=temp_p*0.5,temp_n*0.5
        finalscore.append(temp_n/(temp_n+temp_p))
    dataset['Topsis Score'] =finalscore
    
    #calculating rank
    dataset['Rank'] = (dataset['Topsis Score'].rank(method='max',ascending=False))
    dataset=dataset.astype({"Rank":int})
    
    #Writing the csv
    dataset.to_csv(sys.argv[4],index=False)
    
    
if __name__=="__main__":
    #print(sys.argv)
     main()
    



from distutils.core import setup
setup(
  name = 'Topsis_Drishti_102016102',         # How you named your package folder (MyLib)
  packages = ['Topsis_Drishti_102016102'],   # Chose the same as "name"
  version = '0.3',      # Start with a small number and increase it with every change you make
  license='MIT',        # Chose a license from here: https://help.github.com/articles/licensing-a-repository
  description = 'topsis',   # Give a short description about your library
  author = 'Drishti bhatia',                   # Type in your name
  author_email = 'drishtibhatia2020@gmail.com',      # Type in your E-Mail
      # I explain this later on
  long_description='''Project description
TOPSIS
Technique for Order Preference by Similarity to Ideal Solution (TOPSIS)
Technique for Order Preference by Similarity to Ideal Solution (TOPSIS) came in the 1980s as a multi-criteria-based decision-making method. TOPSIS chooses the alternative of shortest the Euclidean distance from the ideal solution and greatest distance from the negative ideal solution.

TOPSIS is a way to allocate the ranks on basis of the weights and impact of the given factors:.

Weights mean how much a given factor should be taken into consideration
Impact means that a given factor has a positive or negative impact.
This tool allows you to calculate the topsis ranking and save the results in the form of a csv (Comma Seperated Value) file.

Installing Package
pip install Topsis-Drishti-102016102
Using the TOPSIS tool
Create a script by importing the package and just calling the TOPSIS function.
import importlib
topsis=importlib.import_module("Topsis_Drishti_1016102")
topsis.TOPSIS()
Run the Script through command line as shown below:
C:/Users/admin> python myscript.py <Data_File_csv> <Weights(Comma_seperated)> <Impacts(Comma_seperated)> <Result_file_csv>
''',
  long_description_content_type='text/markdown',
  keywords = ['topsis'],   # Keywords that define your package best
  install_requires=[            # I get to this in a second
          'pandas',
          'numpy',
      ],
  classifiers=[
    'Development Status :: 3 - Alpha',      # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
    'Intended Audience :: Developers',      # Define that your audience are developers
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',   # Again, pick a license
    'Programming Language :: Python :: 3',      #Specify which pyhton versions that you want to support
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
  ],
)
